# frozen_string_literal: true

require_relative 'uprofile/version'
require_relative 'uprofile/bluepages/email'
require_relative 'uprofile/bluepages/uid'
require_relative 'uprofile/bluepages/typeahead'
require_relative 'uprofile/bluepages/skills'
require_relative 'uprofile/bluepages/teams'
require_relative 'uprofile/bluepages/all'
require_relative 'uprofile/arguments'

module RailsIbm
  module Uprofile
    class Error < StandardError; end
    DOC_LINK = "Please refer to 'https://github.com/niightly/rails_ibm-uprofile/blob/master/README.md#usage' for more details.'".freeze

    def self.info(options = {})
      args = verify(options)

      if options.include?(:emails) || options.include?(:email)
        RailsIbm::Uprofile::Bluepages::Email.new([options[:emails] || options[:email]].flatten).results(**args.filter(%i[parsed remap]))
      elsif options.include?(:uids) || options.include?(:uid)
        RailsIbm::Uprofile::Bluepages::Uid.new([options[:uids] || options[:uid]].flatten).results(**args.filter(%i[parsed remap]))
      elsif options.include?(:query)
        RailsIbm::Uprofile::Bluepages::Typeahead.new(options[:query], **default_params(options)).results(**args.filter(%i[parsed remap detailed array]))
      end
    end

    def self.skills(options = {})
      args = verify(options, single: true)
      RailsIbm::Uprofile::Bluepages::Skills.new(options[:uid] || options[:email]).results
    end

    def self.teams(options = {})
      args = verify(options, single: true)
      RailsIbm::Uprofile::Bluepages::Teams.new(options[:uid] || options[:email]).results
    end

    def self.all(options = {})
      args = verify(options, single: true)
      RailsIbm::Uprofile::Bluepages::All.new(options[:uid] || options[:email]).results
    end

    private

    def self.verify(options, single: false)
      args = RailsIbm::Uprofile::Arguments.new(params(options, single))
      args.validate

      args
    end

    def self.default_params(options)
      { serial: options[:serial] || '000000', limit: options[:limit] || 10, start: options[:start] || 0 }
    end

    def self.params(options, single)
      return options unless single
      options.tap { |hs| %i[uids emails].each { |k| hs.delete(k) } }
    end
  end
end
