require 'httparty'

module RailsIbm
  module Uprofile
    class Arguments
      def initialize(options)
        @options = options
      end

      def filter(allowed, options = nil)
        (options || @options).select {|k,v| allowed.include?(k) }
      end

      def validate
        raise SyntaxError.new description: "Invalid Argument. #{DOC_LINK}" if invalid_options
        raise SyntaxError.new description: "More than one required Argument. #{DOC_LINK}" if multiple
        raise SyntaxError.new description: "Missing required Argument. #{DOC_LINK}" if missing
        raise TypeError.new description: "Invalid Argument type. #{DOC_LINK}" if invalid_type || unmatch_type

        true
      end

      private

      def value
        @options[:uids] || @options[:emails] || @options[:uid] || @options[:email] || @options[:query]
      end

      def missing
        value == nil
      end

      def multiple
        allowed = %i[uid uids email emails query]
        @options.keys.select { |k| allowed.include?(k) }.size > 1
      end

      def invalid_type
        config = { type: Array, value: value }
        config[:type] = String if @options.include?(:uid) || @options.include?(:email) || @options.include?(:query)

        !config[:value].is_a?(config[:type])
      end

       def unmatch_type
        entry = value
        entry = value.first if value.is_a? Array

        return is_valid_email?(entry) if @options.include?(:uid) || @options.include?(:uids)
        return !is_valid_email?(entry) if @options.include?(:email) || @options.include?(:emails)
        false
      end

      def invalid_options
        !@options.is_a? Hash
      end

      def is_valid_email?(email)
        (email =~ RailsIbm::Uprofile::EMAIL_REGEX) != nil
      end

    end
  end
end
