require 'httparty'
require_relative 'mapper'
require_relative '../async'

module RailsIbm
  module Uprofile
    module Bluepages
      class Fetch
        def initialize(uri:)
          @uri = uri
        end

        def execute(query:)
          @query = query

          return HTTParty.get("#{@uri}/#{uri_params(query)}", headers: headers) unless multiple_search
          RailsIbm::Uprofile::Async.Requests chunks(query).map { |c| "#{@uri}/#{uri_params(c)}" }, headers
        end

        private

        def multiple_search
          @query.is_a?(Array) && @query.size >= 2
        end

        def uri_params(users)
          return users.join(',') if multiple_search
          "#{users.first}/profile"
        end

        def encode_uri(users)
          "#{@uri}/#{ERB::Util.url_encode(uri_params(users))}"
        end

        def chunks(users)
          partial = [[]]
          users.each do |u|
            partial.last.push(u)
            partial.push([]) if encode_uri(partial.last).length > 5000
          end

          partial
        end

        def headers
          {
            'Content-Type': 'application/json',
            accept: 'application/json'
          }
        end
      end
    end
  end
end