require_relative 'search'
require_relative 'email'
require_relative "../version"

module RailsIbm
  module Uprofile
    module Bluepages
      class Typeahead < Search
        def initialize(query, serial: '000000', limit: 10, start: 0)
          uri = "#{RailsIbm::Uprofile::BLUEPAGES_V1_URI}?userId=#{serial}&rows=#{limit}&start=#{start}&query=#{query}&searchConfig=typeahead_search"
          @response = HTTParty.get(uri, header: headers, format: :plain)
        end

        def results(options)
          search_results(options)
        end

        private

        def headers
          {
            'Content-Type': 'application/json',
            accept: 'application/json'
          }
        end

        def search_type
          'typeahead'
        end

        def search_results(options)
          results = JSON.parse @response, symbolize_names: true
          return [] if results == nil

          return results unless default_values(options)[:parsed]

          results = parse_user results, %i[results], remap: default_values(options)[:remap]
          return results unless default_values(options)[:detailed]

          emails = [results].flatten.map { |u| u[:uid] }
          results = Uid.new(emails).results(parsed: default_values(options)[:parsed], remap: default_values(options)[:remap])

          return results.map{ |_k,v| v } if default_values(options)[:array]
          results
        end

        def default_values(options)
          default_params = { parsed: true, detailed: false, remap: false, array: true }
          default_params.merge(options)
        end
      end
    end
  end
end
